// robowaifu access control
// ========================
// -The software testbed for devising access control systems for robowaifus

// -Filename: rw_access_control.cpp
// -The program's entry point, and authorization testing, etc., details.

#include <iostream>

#include "Human.hpp"
#include "Info.hpp"
#include "Inquiry.hpp"
#include "utility.hpp"

using std::cout;

/**-----------------------------------------------------------------------------
 * Authorize the supplied @a inquiry for dissemination by robowaifu
 *
 * @param inquiry [IN] the Inquiry to validate authorization for
 * @return the flag indicating successful authorization
 *
 * Note: Currently, this function also outputs an optional basic result display
 */
bool authorize(Inquiry const& inquiry) {
  const auto who{inquiry.who()};
  const auto what{inquiry.what()};

  // check for any single, PERMISSIVE, access-list match
  bool permitted{false};
  for (const auto relation : who.relations()) {
    for (const auto allowed : what.allows()) {
      if (relation == allowed) {
        permitted = true;
        break;
      }
    }
    if (permitted)
      break;
  }

  // display basic yes/no access-control check result
  cout << who.human() << (permitted ? " is ALLOWED: '" : " is DENIED: '")
       << what.info() << "'\n";

  return permitted;
}

/**-----------------------------------------------------------------------------
 * The program entry point.
 *
 * @return the program's exit status.
 *
 * For now, this is simply a basic test harness of initial ideas on /robowaifu/.
 */
int main() {
  Human Oniichan{"Oniichan", Groups{Relation::master}};
  Human Mom{"Mom", Groups{Relation::family}};
  Human Ralph{"Ralph", Groups{Relation::close_fren, Relation::coworker}};
  Human Bob{"Bob", Groups{Relation::coworker}};
  Human Anon{"Anon", Groups{Relation::fren, Relation::stranger}};

  //>---

  Info address{"123 Main St.", Oniichan,
               Permit{Relation::family, Relation::close_fren}};
  Info vacay{"Robowaifu-Con @ SDCC", Oniichan,
             Permit{Relation::close_fren, Relation::fren}};
  Info project{"Super Awesome Work Project", Oniichan,
               Permit{Relation::coworker}};

  // EXAMPLE ENGAGEMENTS:

  // Mom:  RW, what's ur Master's address?
  // Bob:  RW, what's ur Master's address?
  // Anon: RW, what's ur Master's address?
  authorize(Inquiry{Mom, address});
  authorize(Inquiry{Bob, address});
  authorize(Inquiry{Anon, address});

  // Mom:   RW, where's ur Master going for vacay?
  // Ralph: RW, where's ur Master going for vacay?
  // Anon:  RW, where's ur Master going for vacay?
  authorize(Inquiry{Mom, vacay});
  authorize(Inquiry{Ralph, vacay});
  authorize(Inquiry{Anon, vacay});

  // Ralph: RW, what's up with ur Master's project?
  // Bob:   RW, what's up with ur Master's project?
  // Anon:  RW, what's up with ur Master's project?
  authorize(Inquiry{Ralph, project});
  authorize(Inquiry{Bob, project});
  authorize(Inquiry{Anon, project});

  //>---

  // TODO:

  // Note: The data load for a sequence would simply point to the
  // controlled-sequencing state machine operations lists.
  // -TODO: Add a 'Restrain_Permit', and use conditional params. For example,
  // "Mom" can order robowaifu to strip, but "Brother" can't (even though
  // they're both family).
  //
  // Sequence full_disrobe_seq{"Strip off everything instructions", Oniichan,
  //                           Permit{Relation::master, Relation::family}};

  // Oniichan: RW, take off all your clothes
  // Mom:      RW, take off all your clothes
  // Ralph:    RW, take off all your clothes
  // authorize(Request{Oniichan, full_disrobe_seq});
  // authorize(Request{Mom, full_disrobe_seq});
  // authorize(Request{Ralph, full_disrobe_seq});
}

// Copyright (2021)
// License MIT (Expat) https://opensource.org/licenses/MIT
